#!/usr/bin/perl
###############################################################################
#  Gets the /proc/acpi/alarm  or clear string from vdr and send it to the S100 atmega8 wakeup board
# Commandline arguments:
# 1. Wakeuptime  as ACPI Time String (2006-02-09 23:05:00) or UNIX Systemdate (1141941900)
# or via STDIN
# 
# 25.03.2008  
# changes
# 
# Michael Albert
# Comments to michael@albert-hetzles.de
# 1. FCN forever http://www.fcn.de
###############################################################################
use Time::Local;
use File::Basename;
use Sys::Syslog;

# S100 Serial port
$sSerialPort="/dev/ttyS0";
#$sSerialPort="/tmp/ATx.txt";
# Script messages to syslog
$bUseSysLog=1;
# Set system time
$bSetSystemTime=0;
# Time Servers
@aTimeServer=("ptbtime1.ptb.de","ptbtime2.ptb.de");
# ntpbin
$sNTPDate="sudo /usr/sbin/ntpdate -u ";
# Setup /InitSerialPort
$bSetupSerial=1;
# stty exe string, BAUD 2400, Bits 8, Parity None, 1 Stopbit, noflowcontrol, no hardware handshake
$ssttyBin="stty -F $sSerialPort 2400 cs8 -cstopb -crtscts -ixon -ixoff";

# $sArg=Variable to store first command line parameter
# $iACPIStartTime=Variable to store in systemtime (1141941900)
my ($sArg,$iACPIStartTime,$sMessage);
$sMessage=" Starting VDR wakeup...";
print $sMessage."\n";
($bUseSysLog) && &fWriteSyslog($sMessage); 
# Set time
if ($bSetSystemTime){
	foreach (@aTimeServer){
		$sMessage="Set systemtime from ntpserver $_\n";
		print $sMessage."\n";
		($bUseSysLog) && &fWriteSyslog($sMessage); 
		$iResult=system($sNTPDate.$_);
		if ($iResult){
			$sMessage="Error while setting systemtime $iResult\n";
			print $sMessage."\n";
			($bUseSysLog) && &fWriteSyslog($sMessage); 
		}
	}
}


# Getting parameter
if (@ARGV[0]) {
	# Get command line parameter
	$sArg=@ARGV[0];
}
else{
	# Or from stdin
	$sArg=<STDIN>;
}
# cut \n 
chomp($sArg);
# Check if format is already UNIX systemtime
if ($sArg=~/^[0-9]+$/){
	# Yes, cool
	$sMessage=" Get UNIX system time \($sArg\) via command line.";
	print $sMessage."\n";
	($bUseSysLog) && &fWriteSyslog($sMessage); 
	$iACPIStartTime=$sArg;
}
else{
	# No, we must convert it.
	$sMessage=" Get /proc/acpi/alarm formated string \($sArg\) via command line.";
	print $sMessage."\n";
	($bUseSysLog) && &fWriteSyslog($sMessage); 
	# Convert it to system time format. 2006-02-09 23:05:00 -> 1141941900
	$iACPIStartTime=&fGetSystemTimeFromACPIStartTime($sArg);	
}



if ($iACPIStartTime){
	($bSetupSerial) && system($ssttyBin);
	my ($iTimeNow,$iTimeDiff);
	# print "ACPI Time: $iACPIStartTime\n";
	# Calculating difference between Starttime and now in seconds
	$iTimeDiff=$iACPIStartTime-time;
	if ($iTimeDiff>0){
		$sMessage=" Starting VDR in $iTimeDiff seconds.";
		print $sMessage."\n";
		($bUseSysLog) && &fWriteSyslog($sMessage); 
		# send command string to Board via serial port
		#open(hSTTY,">$sSerialPort")||die "Can't open $sSerialPort\n";
		print 'echo -e "\\rATS'.$iTimeDiff.'\\r" > '.$sSerialPort;
		system('echo -e "\\rATS'.$iTimeDiff.'\\r" > '.$sSerialPort);
		#close(hSTTY);
	}
	else{
		# Think positive :-)
		$iTimeDiff*=-1;
		$sMessage=" VDR starting time is $iTimeDiff seconds in the past\! Resetting timer.";
		warn $sMessage."\n"; 
		($bUseSysLog) && &fWriteSyslog($sMessage); 
		system('echo -e "\\rATC\\r" > '.$sSerialPort);
	}
}
else{
	warn " Unknown command or time format\n";
}


###############################################################################
# 1. Parameter ACPI Time (Format: 2006-02-09 23:05:00)
# Return systemtime
sub fGetSystemTimeFromACPIStartTime(){
	my ($sACPIStartTime,$sDate,$sTime,$sSec,$sMin,$sHour,$sDay,$sMon,$sYear);
	$sACPIStartTime=shift;
	($sDate,$sTime)=split(/ /,$sACPIStartTime);
	($sHour,$sMin,$sSec)=split(/\:/,$sTime);
	($sYear,$sMon,$sDay)=split(/\-/,$sDate);
	 # print"$sSec,$sMin,$sHour,$sDay,$sMon,$sYear\n";
	 # timelocal wants Month from 0..11
	 $sMon-=1;
	# We try to convert it
	eval{
		$iACPISysTime = timelocal($sSec,$sMin,$sHour,$sDay,$sMon,$sYear);
	};
	# successfully? 
	if ($@){
		# No
		my $sErrorMessage;
		$sErrorMessage=$@;
		chomp($sErrorMessage);
		warn " ERROR $sErrorMessage.\n Cannot convert \"$sACPIStartTime\". Wrong format or incorrect date.\n";
		($bUseSysLog) && &fWriteSyslog(" ERROR $sErrorMessage.\n Cannot convert \"$sACPIStartTime\". Wrong format or incorrect date.\n");
		return(0);
	}
	else{
		# Yes
		return($iACPISysTime);
	}
}

# 1. Parameter Text write to syslog
sub fWriteSyslog(){
    my $sText;
    $sText = @_[0];
    openlog($program . basename($0), 'pid', 'user');
    syslog('info', $sText);
    #syslog('mail|warning', 'this is a better test: %d', time);
    closelog();
}